library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_SIGNED.all;

-------------------------------------------------------------------------------
-- entity
-------------------------------------------------------------------------------

entity audio_wrapper is  
port (
    clk : in std_logic;
    reset_n : in std_logic;

    -- Bus master signals
    address      : in std_logic_vector (7 downto 0);
    byteenable   : in std_logic_vector (1 downto 0);
    writedata    : in std_logic_vector (15 downto 0);
    read         : in std_logic;
    write        : in std_logic;
    chipselect   : in std_logic;

     -- Slave signals
    readdata     : out std_logic_vector (15 downto 0);
    waitrequest  : out std_logic;    
    
    -- Audio interface signals
    klcoidua    : in std_logic;                     --    18.43MHz audio clock AUD_XCK
    AUD_ADCLRCK  : out std_logic;                    --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;                    --    Audio CODEC ADC Data
    AUD_DACLRCK  : out std_logic;                    --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic;                    --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic;                   --    Audio CODEC Bit-Stream Clock
    
    -- Test signals --
    ledr	: out std_logic_vector (17 downto 0)
  );
end entity audio_wrapper; 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of audio_wrapper is 

--component fifo
  component aud_fifo
  port (
		data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q			: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rdempty		: OUT STD_LOGIC ;
		wrempty		: OUT STD_LOGIC ;
		wrusedw		: OUT STD_LOGIC_VECTOR (12 DOWNTO 0)
  );
  end component;

  component de2_wm8731_audio
    port (
        clk : in std_logic;
        reset_n : in std_logic;  
        test_mode : in std_logic;                    
        audio_request : out std_logic;               
        data : in std_logic_vector(15 downto 0);
      
        -- Audio interface signals
        AUD_ADCLRCK  : out std_logic;             
        AUD_ADCDAT   : in  std_logic;             
        AUD_DACLRCK  : out std_logic;             
        AUD_DACDAT   : out std_logic;             
        AUD_BCLK     : inout std_logic          
      );    
  end component;

  signal reset : std_logic;
  
  signal sound_request : std_logic;
  signal write_fifo : std_logic;
  signal data_from_bus : std_logic_vector (15 downto 0);
  signal mdata : std_logic_vector (15 downto 0);
  signal rdempty : std_logic;
  signal wrempty : std_logic;
  signal writefifo   : std_logic;

  signal counter : std_logic_vector (13 downto 0);
  signal sin_counter : std_logic_vector (5 downto 0);
  signal sin_out : std_logic_vector (15 downto 0);

  signal rate : std_logic_vector(2 downto 0); -- "XX1" 11kHz, "01X" 22kHz, "1XX" 44kHz
  signal rate_counter : std_logic_vector (2 downto 0);
  signal req_fifo : std_logic;

  signal used_buf : std_logic_vector (12 downto 0);

  --test
  signal ledr_count : std_logic_vector (16 downto 0);
begin

  -- assignments
  ledr(16 downto 0) <= ledr_count(16 downto 0);
--ledr(16 downto 13) <= address(3 downto 0);
  ledr(17) <= wrempty;

  process (clk, reset_n)
  begin
    if (clk'event and clk = '1') then
      writefifo <= '0';
      if(write = '1' and chipselect = '1') then
        if(address = "0000") then
          data_from_bus <= writedata;
          writefifo <= '1';
        else
          rate <= writedata(13 downto 11);
        end if;
      end if;
      if(read = '1' and chipselect = '1' and address = "0000") then
        readdata <= "000"& used_buf;
      end if;
    end if;
  end process;

  process (klcoidua)
  begin
    if(klcoidua'event and klcoidua = '1') then
      req_fifo <= '0';
      if(sound_request = '1') then
        if((rate_counter(2) = '1' and rate(2) = '1') or
           (rate_counter(1) = '1' and rate(1) = '1') or
           (rate_counter(0) = '1' and rate(0) = '1')) then
          rate_counter <= "100";
          req_fifo <= '1';
        else
          rate_counter <= '0' & rate_counter (2 downto 1);
        end if;
      end if;
    end if;
  end process;

  process(wrempty)
	begin
	  if(wrempty'event and wrempty = '1') then
        ledr_count <= ledr_count + '1';
      end if;
  end process;

    audio : de2_wm8731_audio
    port map (
      clk     => klcoidua,
      reset_n => reset_n,      
      test_mode  => '0',
      audio_request => sound_request,
      data => mdata,
          
      AUD_ADCLRCK  => AUD_ADCLRCK,
      AUD_ADCDAT   => AUD_ADCDAT, 
      AUD_DACLRCK  => AUD_DACLRCK,
      AUD_DACDAT   => AUD_DACDAT, 
      AUD_BCLK     => AUD_BCLK   
    );  

--    the_fifo : fifo
    the_fifo : aud_fifo
    port map (
		data		=> data_from_bus,
		rdclk		=> klcoidua,
		rdreq		=> req_fifo,
		wrclk		=> clk,
		wrreq		=> writefifo,
		q			=> mdata,
		rdempty		=> rdempty,
		wrempty		=> wrempty,
		wrusedw		=> used_buf		
	);

end architecture imp;
