library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--library UNISIM;
--use UNISIM.VComponents.all;

entity de2_wm8731_audio is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    test_mode : in std_logic;       --    Audio CODEC controller test mode
    audio_request : out std_logic;  --    Audio controller request new data
    data : in std_logic_vector(15 downto 0);
  
    -- Audio interface signals
    AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic;   --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
end  de2_wm8731_audio;


architecture Behavioral of  de2_wm8731_audio is     

    signal lrck : std_logic;
    signal bclk : std_logic;
    signal xck : std_logic;
    
    signal lrck_divider : std_logic_vector (7 downto 0); 
    signal bclk_divider : std_logic_vector (3 downto 0);
    
    signal set_bclk : std_logic;
    signal set_lrck : std_logic;
    signal clr_bclk : std_logic;
    signal lrck_lat : std_logic;
    
    signal shift_out : std_logic_vector ( 15 downto 0);

    signal sin_out : std_logic_vector ( 15 downto 0);
    signal sin_counter : std_logic_vector ( 5 downto 0);
    

begin
  
    -- LRCK divider 
    -- Audio chip main clock is 18.432MHz / Sample rate 48KHz
    -- Divider is 18.432 MHz / 48KHz = 192 (X"C0")
    -- Left justify mode set by I2C controller
    
    process(clk, reset_n)
    begin
        if ( reset_n = '0' ) then 
            lrck_divider <= (others => '0');
        elsif ( clk'event and clk='1' ) then
            if ( lrck_divider = X"BF")  then        -- "C0" minus 1
                lrck_divider <= X"00";
            else 
                lrck_divider <= lrck_divider + '1';
            end if;
        end if;
    end process;

    process(clk, reset_n)
    begin
        if ( reset_n = '0' ) then 
            bclk_divider <= (others => '0');
        elsif ( clk'event and clk='1' ) then
            if ( bclk_divider = X"B" or set_lrck = '1')  then  
                bclk_divider <= X"0";
            else 
                bclk_divider <= bclk_divider + '1';
            end if;
        end if;
    end process;
    
  
    process ( lrck_divider ) 
    begin
        if ( lrck_divider = X"BF") then
            set_lrck <= '1';
        else
            set_lrck <= '0';
        end if;
    end process;
    
    process ( clk, reset_n) 
    begin
        if ( reset_n = '0') then
            lrck <= '0';
        elsif ( clk 'event and clk = '1') then 
            if ( set_lrck = '1') then 
                lrck <= not lrck;
            end if;
        end if;
    end process;
    
    -- BCLK divider    
    process ( bclk_divider ) 
    begin
        if ( bclk_divider (3 downto 0) = "0101") then
            set_bclk <= '1';                      
        else
            set_bclk <= '0';
        end if;      

        if ( bclk_divider (3 downto 0) = "1011") then
            clr_bclk <= '1';                      
        else
            clr_bclk <= '0';
        end if;              
    end process;
    
    process ( clk, reset_n) 
    begin
        if ( reset_n = '0') then
            bclk <= '0';
        elsif ( clk 'event and clk = '1') then 
            if ( set_lrck = '1' or clr_bclk = '1') then
                bclk <= '0';
            elsif ( set_bclk = '1') then 
                bclk <= '1';
            end if;
        end if;
    end process;

    -- Audio data shift output
    process ( clk, reset_n) 
    begin
        if ( reset_n = '0') then
            shift_out <= (others => '0');
        elsif ( clk 'event and clk = '1') then 
            if ( set_lrck = '1') then
                if ( test_mode = '1') then 
                    shift_out <= sin_out;
                else 
                    shift_out <= data;
                end if;
            elsif ( clr_bclk = '1') then 
                shift_out <= shift_out (14 downto 0) & '0';
            end if;
        end if;
    end process;

    -- Audio outputs
    
    AUD_ADCLRCK  <= lrck;          
    AUD_DACLRCK  <= lrck;          
    AUD_DACDAT   <= shift_out(15); 
    AUD_BCLK     <= bclk;          

    -- Self test with Sin wave
    
    process(clk, reset_n)
    begin
        if ( reset_n = '0' ) then 
            sin_counter <= (others => '0');
        elsif ( clk'event and clk='1' ) then
            if ( lrck_lat = '1' and lrck = '0')  then  
                if (sin_counter = "101111") then 
                    sin_counter <= "000000";
                else  
                    sin_counter <= sin_counter + '1';
                end if;
            end if;
        end if;
    end process;

    process(clk)
    begin
        if ( clk'event and clk='1' ) then
            lrck_lat <= lrck;
        end if;
    end process;

    process (clk) 
    begin
        if ( clk'event and clk = '1') then 
            if ( lrck_lat = '1' and lrck = '0') then
                audio_request <= '1';
            else 
                audio_request <= '0';
            end if;
        end if;
    end process;

    process ( sin_counter ) 
    begin
        case sin_counter is 
            when "000000" => sin_out <= X"0000";
            when "000001" => sin_out <= X"10b4";
            when "000010" => sin_out <= X"2120";
            when "000011" => sin_out <= X"30fb";
            when "000100" => sin_out <= X"3fff";
            when "000101" => sin_out <= X"4deb";
            when "000110" => sin_out <= X"5a81";
            when "000111" => sin_out <= X"658b";
            when "001000" => sin_out <= X"6ed9";
            when "001001" => sin_out <= X"7640";
            when "001010" => sin_out <= X"7ba2";
            when "001011" => sin_out <= X"7ee6";
            when "001100" => sin_out <= X"7fff";
            when "001101" => sin_out <= X"7ee6";
            when "001110" => sin_out <= X"7ba2";
            when "001111" => sin_out <= X"7640";
            when "010000" => sin_out <= X"6ed9";
            when "010001" => sin_out <= X"658b";
            when "010010" => sin_out <= X"5a81";
            when "010011" => sin_out <= X"4deb";
            when "010100" => sin_out <= X"3fff";
            when "010101" => sin_out <= X"30fb";
            when "010110" => sin_out <= X"2120";
            when "010111" => sin_out <= X"10b4";
            when "011000" => sin_out <= X"0000";
            when "011001" => sin_out <= X"ef4b";
            when "011010" => sin_out <= X"dee0";
            when "011011" => sin_out <= X"cf05";
            when "011100" => sin_out <= X"c001";
            when "011101" => sin_out <= X"b215";
            when "011110" => sin_out <= X"a57e";
            when "011111" => sin_out <= X"9a74";
            when "100000" => sin_out <= X"9127";
            when "100001" => sin_out <= X"89bf";
            when "100010" => sin_out <= X"845d";
            when "100011" => sin_out <= X"8119";
            when "100100" => sin_out <= X"8000";
            when "100101" => sin_out <= X"8119";
            when "100110" => sin_out <= X"845d";
            when "100111" => sin_out <= X"89bf";
            when "101000" => sin_out <= X"9127";
            when "101001" => sin_out <= X"9a74";
            when "101010" => sin_out <= X"a57e";
            when "101011" => sin_out <= X"b215";
            when "101100" => sin_out <= X"c000";
            when "101101" => sin_out <= X"cf05";
            when "101110" => sin_out <= X"dee0";
            when "101111" => sin_out <= X"ef4b";
            when others => sin_out <= X"0000";
        end case;
    end process;
      

end architecture;
